/*
 * Severalnines Tools
 * Copyright (C) 2016  Severalnines AB
 *
 * This file is part of s9s-tools.
 *
 * s9s-tools is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Foobar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Foobar. If not, see <http://www.gnu.org/licenses/>.
 */
#include "ut_s9sserver.h"

#include "s9suser.h"
#include "s9sserver.h"
#include <cstdio>
#include <cstring>

//#define DEBUG
#include "s9sdebug.h"

UtS9sServer::UtS9sServer()
{
    S9S_DEBUG("");
}

UtS9sServer::~UtS9sServer()
{
}

bool
UtS9sServer::runTest(
        const char *testName)
{
    bool retval = true;

    PERFORM_TEST(testConstruct,   retval);
    PERFORM_TEST(testProperties,  retval);

    return retval;
}

/**
 */
bool
UtS9sServer::testConstruct()
{
    S9sServer server;

    S9S_COMPARE(server.className(), "CmonHost");
    return true;
}

bool
UtS9sServer::testProperties()
{
    S9sVariantMap theMap;
    S9sServer     server;
    const S9sString jsonString = 
"   {\n"
"        'acl': 'user::rwx,group::rw-,other::---',\n"
"        'cdt_path': '/',\n"
"        'class_name': 'CmonLxcServer',\n"
"        'clusterid': 0,\n"
"        'connected': true,\n"
"        'containers': [ \n"
"        {\n"
"            'acl': 'user::rwx,group::rw-,other::---',\n"
"            'alias': 'bestw_controller',\n"
"            'architecture': 'x86_64',\n"
"            'autostart': false,\n"
"            'cdt_path': '/core1/containers',\n"
"            'class_name': 'CmonContainer',\n"
"            'cluster_id': 0,\n"
"            'configfile': '/var/lib/lxc/bestw_controller/config',\n"
"            'hostname': 'bestw_controller',\n"
"            'is_claimed': false,\n"
"            'memory_limit': 17179869184,\n"
"            'owner_group_id': 4,\n"
"            'owner_group_name': 'testgroup',\n"
"            'owner_user_id': 3,\n"
"            'owner_user_name': 'pipas',\n"
"            'parent_server': 'core1',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1',\n"
"            'root_fs_path': '/var/lib/lxc/bestw_controller/rootfs',\n"
"            'status': 'STOPPED',\n"
"            'subnet': \n"
"            {\n"
"                'az': 'region1',\n"
"                'cidr': '192.168.0.4/16',\n"
"                'class_name': 'CmonSubnet',\n"
"                'id': 'core1-br0',\n"
"                'name': 'br0',\n"
"                'provider': 'lxc',\n"
"                'region': 'region1',\n"
"                'state': 'available',\n"
"                'vpc_id': 'vpc-region1'\n"
"            },\n"
"            'template': 'ubuntu',\n"
"            'type': 'lxc',\n"
"            'version': 1\n"
"        }, \n"
"        {\n"
"            'acl': 'user::rwx,group::rw-,other::---',\n"
"            'alias': 'dns1',\n"
"            'architecture': 'x86_64',\n"
"            'autostart': true,\n"
"            'cdt_path': '/core1/containers',\n"
"            'class_name': 'CmonContainer',\n"
"            'cluster_id': 0,\n"
"            'configfile': '/var/lib/lxc/dns1/config',\n"
"            'hostname': '192.168.0.2',\n"
"            'ip': '192.168.0.2',\n"
"            'ipv4_addresses': [ '192.168.0.2' ],\n"
"            'is_claimed': false,\n"
"            'memory_limit': 4294967296,\n"
"            'network': \n"
"            {\n"
"                'private_ip': [ '192.168.0.2' ],\n"
"                'public_ip': [  ]\n"
"            },\n"
"            'os_version': \n"
"            {\n"
"                'class_name': 'CmonOsVersion',\n"
"                'codename': 'xenial',\n"
"                'name': 'ubuntu',\n"
"                'release': '16.04',\n"
"                'type': 'debian'\n"
"            },\n"
"            'owner_group_id': 4,\n"
"            'owner_group_name': 'testgroup',\n"
"            'owner_user_id': 3,\n"
"            'owner_user_name': 'pipas',\n"
"            'parent_server': 'core1',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1',\n"
"            'root_fs_path': '/var/lib/lxc/dns1/rootfs',\n"
"            'status': 'RUNNING',\n"
"            'subnet': \n"
"            {\n"
"                'az': 'region1',\n"
"                'cidr': '192.168.0.4/16',\n"
"                'class_name': 'CmonSubnet',\n"
"                'id': 'core1-br0',\n"
"                'name': 'br0',\n"
"                'provider': 'lxc',\n"
"                'region': 'region1',\n"
"                'state': 'available',\n"
"                'vpc_id': 'vpc-region1'\n"
"            },\n"
"            'template': 'ubuntu',\n"
"            'type': 'lxc',\n"
"            'version': 1\n"
"        }, \n"
"        {\n"
"            'acl': 'user::rwx,group::rw-,other::---',\n"
"            'alias': 'mqtt',\n"
"            'architecture': 'x86_64',\n"
"            'autostart': true,\n"
"            'cdt_path': '/core1/containers',\n"
"            'class_name': 'CmonContainer',\n"
"            'cluster_id': 0,\n"
"            'configfile': '/var/lib/lxc/mqtt/config',\n"
"            'hostname': '192.168.0.5',\n"
"            'ip': '192.168.0.5',\n"
"            'ipv4_addresses': [ '192.168.0.5' ],\n"
"            'is_claimed': false,\n"
"            'memory_limit': 4294967296,\n"
"            'network': \n"
"            {\n"
"                'private_ip': [ '192.168.0.5' ],\n"
"                'public_ip': [  ]\n"
"            },\n"
"            'os_version': \n"
"            {\n"
"                'class_name': 'CmonOsVersion',\n"
"                'codename': 'xenial',\n"
"                'name': 'ubuntu',\n"
"                'release': '16.04',\n"
"                'type': 'debian'\n"
"            },\n"
"            'owner_group_id': 4,\n"
"            'owner_group_name': 'testgroup',\n"
"            'owner_user_id': 3,\n"
"            'owner_user_name': 'pipas',\n"
"            'parent_server': 'core1',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1',\n"
"            'root_fs_path': '/var/lib/lxc/mqtt/rootfs',\n"
"            'status': 'RUNNING',\n"
"            'subnet': \n"
"            {\n"
"                'az': 'region1',\n"
"                'cidr': '192.168.0.4/16',\n"
"                'class_name': 'CmonSubnet',\n"
"                'id': 'core1-br0',\n"
"                'name': 'br0',\n"
"                'provider': 'lxc',\n"
"                'region': 'region1',\n"
"                'state': 'available',\n"
"                'vpc_id': 'vpc-region1'\n"
"            },\n"
"            'template': 'ubuntu',\n"
"            'type': 'lxc',\n"
"            'version': 1\n"
"        }, \n"
"        {\n"
"            'acl': 'user::rwx,group::rw-,other::---',\n"
"            'alias': 'node001',\n"
"            'architecture': 'x86_64',\n"
"            'autostart': false,\n"
"            'cdt_path': '/core1/containers',\n"
"            'class_name': 'CmonContainer',\n"
"            'cluster_id': 0,\n"
"            'configfile': '/var/lib/lxc/node001/config',\n"
"            'hostname': '192.168.0.163',\n"
"            'ip': '192.168.0.163',\n"
"            'ipv4_addresses': [ '192.168.0.163' ],\n"
"            'is_claimed': false,\n"
"            'memory_limit': 17179869184,\n"
"            'network': \n"
"            {\n"
"                'private_ip': [ '192.168.0.163' ],\n"
"                'public_ip': [  ]\n"
"            },\n"
"            'os_version': \n"
"            {\n"
"                'class_name': 'CmonOsVersion',\n"
"                'codename': 'xenial',\n"
"                'name': 'ubuntu',\n"
"                'release': '16.04',\n"
"                'type': 'debian'\n"
"            },\n"
"            'owner_group_id': 4,\n"
"            'owner_group_name': 'testgroup',\n"
"            'owner_user_id': 3,\n"
"            'owner_user_name': 'pipas',\n"
"            'parent_server': 'core1',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1',\n"
"            'root_fs_path': '/var/lib/lxc/node001/rootfs',\n"
"            'status': 'RUNNING',\n"
"            'subnet': \n"
"            {\n"
"                'az': 'region1',\n"
"                'cidr': '192.168.0.4/16',\n"
"                'class_name': 'CmonSubnet',\n"
"                'id': 'core1-br0',\n"
"                'name': 'br0',\n"
"                'provider': 'lxc',\n"
"                'region': 'region1',\n"
"                'state': 'available',\n"
"                'vpc_id': 'vpc-region1'\n"
"            },\n"
"            'type': 'lxc',\n"
"            'version': 1\n"
"        }, \n"
"        {\n"
"            'acl': 'user::rwx,group::rw-,other::---',\n"
"            'alias': 'ubuntu',\n"
"            'architecture': 'x86_64',\n"
"            'autostart': false,\n"
"            'cdt_path': '/core1/containers',\n"
"            'class_name': 'CmonContainer',\n"
"            'cluster_id': 0,\n"
"            'configfile': '/var/lib/lxc/ubuntu/config',\n"
"            'hostname': 'ubuntu',\n"
"            'is_claimed': false,\n"
"            'memory_limit': 17179869184,\n"
"            'owner_group_id': 4,\n"
"            'owner_group_name': 'testgroup',\n"
"            'owner_user_id': 3,\n"
"            'owner_user_name': 'pipas',\n"
"            'parent_server': 'core1',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1',\n"
"            'root_fs_path': '/var/lib/lxc/ubuntu/rootfs',\n"
"            'status': 'STOPPED',\n"
"            'subnet': \n"
"            {\n"
"                'az': 'region1',\n"
"                'cidr': '192.168.0.4/16',\n"
"                'class_name': 'CmonSubnet',\n"
"                'id': 'core1-br0',\n"
"                'name': 'br0',\n"
"                'provider': 'lxc',\n"
"                'region': 'region1',\n"
"                'state': 'available',\n"
"                'vpc_id': 'vpc-region1'\n"
"            },\n"
"            'type': 'lxc',\n"
"            'version': 1\n"
"        }, \n"
"        {\n"
"            'acl': 'user::rwx,group::rw-,other::---',\n"
"            'alias': 'www',\n"
"            'architecture': 'x86_64',\n"
"            'autostart': true,\n"
"            'cdt_path': '/core1/containers',\n"
"            'class_name': 'CmonContainer',\n"
"            'cluster_id': 0,\n"
"            'configfile': '/var/lib/lxc/www/config',\n"
"            'hostname': '192.168.0.19',\n"
"            'ip': '192.168.0.19',\n"
"            'ipv4_addresses': [ '192.168.0.19' ],\n"
"            'is_claimed': false,\n"
"            'memory_limit': 4294967296,\n"
"            'network': \n"
"            {\n"
"                'private_ip': [ '192.168.0.19' ],\n"
"                'public_ip': [  ]\n"
"            },\n"
"            'os_version': \n"
"            {\n"
"                'class_name': 'CmonOsVersion',\n"
"                'codename': 'xenial',\n"
"                'name': 'ubuntu',\n"
"                'release': '16.04',\n"
"                'type': 'debian'\n"
"            },\n"
"            'owner_group_id': 4,\n"
"            'owner_group_name': 'testgroup',\n"
"            'owner_user_id': 3,\n"
"            'owner_user_name': 'pipas',\n"
"            'parent_server': 'core1',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1',\n"
"            'root_fs_path': '/var/lib/lxc/www/rootfs',\n"
"            'status': 'RUNNING',\n"
"            'subnet': \n"
"            {\n"
"                'az': 'region1',\n"
"                'cidr': '192.168.0.4/16',\n"
"                'class_name': 'CmonSubnet',\n"
"                'id': 'core1-br0',\n"
"                'name': 'br0',\n"
"                'provider': 'lxc',\n"
"                'region': 'region1',\n"
"                'state': 'available',\n"
"                'vpc_id': 'vpc-region1'\n"
"            },\n"
"            'template': 'ubuntu',\n"
"            'type': 'lxc',\n"
"            'version': 1\n"
"        } ],\n"
"        'disk_devices': [ \n"
"        {\n"
"            'class_name': 'CmonDiskDevice',\n"
"            'device': '/dev/mapper/core1--vg-root',\n"
"            'filesystem': 'ext4',\n"
"            'free_mb': 134490,\n"
"            'mountpoint': '/',\n"
"            'total_mb': 166180\n"
"        }, \n"
"        {\n"
"            'class_name': 'CmonDiskDevice',\n"
"            'device': '/dev/sdc1',\n"
"            'filesystem': 'ext2',\n"
"            'free_mb': 342,\n"
"            'mountpoint': '/boot',\n"
"            'total_mb': 471\n"
"        }, \n"
"        {\n"
"            'class_name': 'CmonDiskDevice',\n"
"            'device': 'cgmfs',\n"
"            'filesystem': 'tmpfs',\n"
"            'free_mb': 0,\n"
"            'mountpoint': '/run/cgmanager/fs',\n"
"            'total_mb': 0\n"
"        }, \n"
"        {\n"
"            'device': '',\n"
"            'is_hardware_storage': true,\n"
"            'model': 'KINGSTON SUV400S',\n"
"            'total_mb': 114474\n"
"        }, \n"
"        {\n"
"            'device': '/dev/sdc',\n"
"            'is_hardware_storage': false,\n"
"            'model': 'LSILOGIC Logical Volume',\n"
"            'total_mb': 170230,\n"
"            'volumes': [ \n"
"            {\n"
"                'description': 'Linux filesystem partition',\n"
"                'device': '/dev/sdc1',\n"
"                'filesystem': 'ext2',\n"
"                'free_mb': 342,\n"
"                'mount_point': '/boot',\n"
"                'total_mb': 487\n"
"            }, \n"
"            {\n"
"                'description': 'Extended partition',\n"
"                'device': '/dev/sdc2',\n"
"                'filesystem': '',\n"
"                'mount_point': '',\n"
"                'total_mb': 169741,\n"
"                'volumes': [ \n"
"                {\n"
"                    'description': 'Linux LVM Physical Volume partition',\n"
"                    'device': '/dev/sdc5',\n"
"                    'filesystem': '',\n"
"                    'mount_point': '',\n"
"                    'total_mb': 169741\n"
"                } ]\n"
"            } ]\n"
"        }, \n"
"        {\n"
"            'device': '',\n"
"            'is_hardware_storage': true,\n"
"            'model': 'KINGSTON SV300S3',\n"
"            'total_mb': 114474\n"
"        }, \n"
"        {\n"
"            'device': '',\n"
"            'is_hardware_storage': true,\n"
"            'model': 'KINGSTON SV300S3',\n"
"            'total_mb': 114474\n"
"        }, \n"
"        {\n"
"            'device': '',\n"
"            'is_hardware_storage': true,\n"
"            'model': 'KINGSTON SV300S3',\n"
"            'total_mb': 114474\n"
"        }, \n"
"        {\n"
"            'device': '',\n"
"            'is_hardware_storage': true,\n"
"            'model': 'FUJITSU MBE2147RC',\n"
"            'total_mb': 0\n"
"        }, \n"
"        {\n"
"            'device': '',\n"
"            'is_hardware_storage': true,\n"
"            'model': 'FUJITSU MBE2147RC',\n"
"            'total_mb': 0\n"
"        }, \n"
"        {\n"
"            'device': '',\n"
"            'is_hardware_storage': true,\n"
"            'model': 'FUJITSU MBE2147RC',\n"
"            'total_mb': 0\n"
"        }, \n"
"        {\n"
"            'device': '',\n"
"            'is_hardware_storage': true,\n"
"            'model': 'FUJITSU MBE2147RC',\n"
"            'total_mb': 0\n"
"        }, \n"
"        {\n"
"            'device': '/dev/sdb',\n"
"            'is_hardware_storage': false,\n"
"            'model': 'LSILOGIC Logical Volume',\n"
"            'total_mb': 556944,\n"
"            'volumes': [ \n"
"            {\n"
"                'description': 'EXT4 volume',\n"
"                'device': '/dev/sdb1',\n"
"                'filesystem': 'ext4',\n"
"                'mount_point': '',\n"
"                'total_mb': 556943\n"
"            } ]\n"
"        }, \n"
"        {\n"
"            'device': '/dev/sda',\n"
"            'is_hardware_storage': false,\n"
"            'model': 'SCSI Disk',\n"
"            'total_mb': 15272.1,\n"
"            'volumes': [ \n"
"            {\n"
"                'description': 'EXT4 volume',\n"
"                'device': '/dev/sda1',\n"
"                'filesystem': 'ext4',\n"
"                'mount_point': '',\n"
"                'total_mb': 15271.1\n"
"            } ]\n"
"        } ],\n"
"        'distribution': \n"
"        {\n"
"            'codename': 'xenial',\n"
"            'name': 'ubuntu',\n"
"            'release': '16.04',\n"
"            'type': 'debian'\n"
"        },\n"
"        'hostId': 1,\n"
"        'hostname': 'core1',\n"
"        'hoststatus': 'CmonHostOnline',\n"
"        'images': [ \n"
"        {\n"
"            'name': 'ubuntu_artful',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'ubuntu_bionic',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'ubuntu_trusty',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'ubuntu_xenial',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'ubuntu_zesty',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'debian_buster',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'debian_jessie',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'debian_sid',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'debian_stretch',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'debian_wheezy',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'alpine_3.4',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'alpine_3.5',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'alpine_3.6',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'alpine_3.7',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'alpine_edge',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'centos_6',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'centos_7',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'fedora_25',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'fedora_26',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'fedora_27',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'opensuse_42.2',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'opensuse_42.3',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'oracle_6',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'oracle_7',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'plamo_5.x',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'plamo_6.x',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'archlinux_current',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'gentoo_current',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        } ],\n"
"        'ip': '192.168.0.4',\n"
"        'last_container_collect': 1524213212,\n"
"        'last_hw_collect': 1524213207,\n"
"        'lastseen': 1524213205,\n"
"        'maintenance_mode_active': false,\n"
"        'max_containers': -1,\n"
"        'max_containers_running': -1,\n"
"        'memory': \n"
"        {\n"
"            'banks': [ \n"
"            {\n"
"                'bank': '0',\n"
"                'name': 'DIMM 800 MHz (1.2 ns)',\n"
"                'size': 4294967296\n"
"            }, \n"
"            {\n"
"                'bank': '1',\n"
"                'name': 'DIMM 800 MHz (1.2 ns)',\n"
"                'size': 4294967296\n"
"            }, \n"
"            {\n"
"                'bank': '2',\n"
"                'name': 'DIMM 800 MHz (1.2 ns)',\n"
"                'size': 4294967296\n"
"            }, \n"
"            {\n"
"                'bank': '3',\n"
"                'name': 'DIMM 800 MHz (1.2 ns)',\n"
"                'size': 4294967296\n"
"            }, \n"
"            {\n"
"                'bank': '4',\n"
"                'name': 'DIMM 800 MHz (1.2 ns)',\n"
"                'size': 4294967296\n"
"            }, \n"
"            {\n"
"                'bank': '5',\n"
"                'name': 'DIMM 800 MHz (1.2 ns)',\n"
"                'size': 4294967296\n"
"            }, \n"
"            {\n"
"                'bank': '6',\n"
"                'name': 'DIMM 800 MHz (1.2 ns)',\n"
"                'size': 4294967296\n"
"            }, \n"
"            {\n"
"                'bank': '7',\n"
"                'name': 'DIMM 800 MHz (1.2 ns)',\n"
"                'size': 4294967296\n"
"            }, \n"
"            {\n"
"                'bank': '0',\n"
"                'name': 'DIMM 800 MHz (1.2 ns)',\n"
"                'size': 4294967296\n"
"            }, \n"
"            {\n"
"                'bank': '1',\n"
"                'name': 'DIMM 800 MHz (1.2 ns)',\n"
"                'size': 4294967296\n"
"            }, \n"
"            {\n"
"                'bank': '2',\n"
"                'name': 'DIMM 800 MHz (1.2 ns)',\n"
"                'size': 4294967296\n"
"            }, \n"
"            {\n"
"                'bank': '3',\n"
"                'name': 'DIMM 800 MHz (1.2 ns)',\n"
"                'size': 4294967296\n"
"            }, \n"
"            {\n"
"                'bank': '4',\n"
"                'name': 'DIMM 800 MHz (1.2 ns)',\n"
"                'size': 4294967296\n"
"            }, \n"
"            {\n"
"                'bank': '5',\n"
"                'name': 'DIMM 800 MHz (1.2 ns)',\n"
"                'size': 4294967296\n"
"            }, \n"
"            {\n"
"                'bank': '6',\n"
"                'name': 'DIMM 800 MHz (1.2 ns)',\n"
"                'size': 4294967296\n"
"            }, \n"
"            {\n"
"                'bank': '7',\n"
"                'name': 'DIMM 800 MHz (1.2 ns)',\n"
"                'size': 4294967296\n"
"            } ],\n"
"            'class_name': 'CmonMemoryInfo',\n"
"            'memory_available_mb': 59788,\n"
"            'memory_free_mb': 50498,\n"
"            'memory_total_mb': 64420,\n"
"            'swap_free_mb': 0,\n"
"            'swap_total_mb': 0\n"
"        },\n"
"        'message': 'Up and running.',\n"
"        'model': 'SUN FIRE X4170 SERVER (4583256-1)',\n"
"        'network_interfaces': [ \n"
"        {\n"
"            'gbits': 1,\n"
"            'link': true,\n"
"            'mac': '00:21:28:76:06:2a',\n"
"            'model': '82575EB Gigabit Network Connection',\n"
"            'name': 'enp1s0f0'\n"
"        }, \n"
"        {\n"
"            'gbits': 1,\n"
"            'link': false,\n"
"            'mac': '00:21:28:76:06:2b',\n"
"            'model': '82575EB Gigabit Network Connection',\n"
"            'name': 'enp1s0f1'\n"
"        }, \n"
"        {\n"
"            'gbits': 1,\n"
"            'link': false,\n"
"            'mac': '00:21:28:76:06:2c',\n"
"            'model': '82575EB Gigabit Network Connection',\n"
"            'name': 'enp7s0f0'\n"
"        }, \n"
"        {\n"
"            'gbits': 1,\n"
"            'link': false,\n"
"            'mac': '00:21:28:76:06:2d',\n"
"            'model': '82575EB Gigabit Network Connection',\n"
"            'name': 'enp7s0f1'\n"
"        }, \n"
"        {\n"
"            'gbits': 0,\n"
"            'link': true,\n"
"            'mac': 'fe:01:55:a0:82:02',\n"
"            'model': '',\n"
"            'name': 'veth2LQXTT'\n"
"        }, \n"
"        {\n"
"            'gbits': 0,\n"
"            'link': false,\n"
"            'mac': '52:54:00:f4:2d:e7',\n"
"            'model': '',\n"
"            'name': 'virbr0-nic'\n"
"        }, \n"
"        {\n"
"            'gbits': 0,\n"
"            'link': true,\n"
"            'mac': 'fe:b1:71:8d:e9:1b',\n"
"            'model': '',\n"
"            'name': 'vethR2EYS4'\n"
"        }, \n"
"        {\n"
"            'gbits': 0,\n"
"            'link': true,\n"
"            'mac': 'fe:75:97:ab:c5:80',\n"
"            'model': '',\n"
"            'name': 'vethOTMW4H'\n"
"        }, \n"
"        {\n"
"            'gbits': 0,\n"
"            'link': true,\n"
"            'mac': 'fe:cf:4b:25:9b:cd',\n"
"            'model': '',\n"
"            'name': 'veth735PI1'\n"
"        }, \n"
"        {\n"
"            'gbits': 0,\n"
"            'ip': '10.0.3.1',\n"
"            'mac': '',\n"
"            'model': '',\n"
"            'name': 'lxcbr0'\n"
"        }, \n"
"        {\n"
"            'gbits': 0,\n"
"            'ip': '192.168.0.4',\n"
"            'mac': '',\n"
"            'model': '',\n"
"            'name': 'br0'\n"
"        }, \n"
"        {\n"
"            'gbits': 0,\n"
"            'ip': '192.168.122.1',\n"
"            'mac': '',\n"
"            'model': '',\n"
"            'name': 'virbr0'\n"
"        } ],\n"
"        'owner_group_id': 4,\n"
"        'owner_group_name': 'testgroup',\n"
"        'owner_user_id': 3,\n"
"        'owner_user_name': 'pipas',\n"
"        'processors': [ \n"
"        {\n"
"            'cores': 4,\n"
"            'cpu_max_ghz': 1.6,\n"
"            'id': 5,\n"
"            'model': 'Intel(R) Xeon(R) CPU L5520 @ 2.27GHz',\n"
"            'siblings': 8,\n"
"            'vendor': 'Intel Corp.'\n"
"        }, \n"
"        {\n"
"            'cores': 4,\n"
"            'cpu_max_ghz': 1.6,\n"
"            'id': 9,\n"
"            'model': 'Intel(R) Xeon(R) CPU L5520 @ 2.27GHz',\n"
"            'siblings': 8,\n"
"            'vendor': 'Intel Corp.'\n"
"        } ],\n"
"        'protocol': 'lxc',\n"
"        'subnets': [ \n"
"        {\n"
"            'az': 'region1',\n"
"            'cidr': '192.168.0.4/16',\n"
"            'class_name': 'CmonSubnet',\n"
"            'id': 'core1-br0',\n"
"            'name': 'br0',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1',\n"
"            'state': 'available',\n"
"            'vpc_id': 'vpc-region1'\n"
"        }, \n"
"        {\n"
"            'az': 'core1-internal',\n"
"            'cidr': '10.0.3.1/24',\n"
"            'class_name': 'CmonSubnet',\n"
"            'id': 'core1-lxcbr0',\n"
"            'name': 'lxcbr0',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1',\n"
"            'state': 'down',\n"
"            'vpc_id': 'vpc-core1-lxcbr0'\n"
"        }, \n"
"        {\n"
"            'az': 'core1-internal',\n"
"            'cidr': '192.168.122.1/24',\n"
"            'class_name': 'CmonSubnet',\n"
"            'id': 'core1-virbr0',\n"
"            'name': 'virbr0',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1',\n"
"            'state': 'down',\n"
"            'vpc_id': 'vpc-core1-virbr0'\n"
"        } ],\n"
"        'templates': [ \n"
"        {\n"
"            'name': 'bestw_controller',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        }, \n"
"        {\n"
"            'name': 'ubuntu',\n"
"            'provider': 'lxc',\n"
"            'region': 'region1'\n"
"        } ],\n"
"        'timestamp': 1524213212,\n"
"        'unique_id': 1,\n"
"        'version': '2.0.8'\n"
"    }\n";

    // Creating a map and setting up the user accordingly.
    S9S_VERIFY(theMap.parse(STR(jsonString)));
    server = theMap;

    S9S_COMPARE(server.className(), "CmonLxcServer");
    S9S_COMPARE(server.name(),      "core1");
    S9S_COMPARE(server.id(),        "1");
    S9S_COMPARE(server.version(),   "2.0.8");
    S9S_COMPARE(server.ipAddress(), "192.168.0.4");
    S9S_COMPARE(server.protocol(),  "lxc");
    S9S_COMPARE(server.status(),    "CmonHostOnline");

    return true;
}


S9S_UNIT_TEST_MAIN(UtS9sServer)



